@@ -1,8 +1,8 @@
 # update data for use in brms functions
 # @param data the data passed by the user
 # @param bterms object of class brmsterms
-# @param na.action function defining how to treat NAs
-# @param drop.unused.levels should unused factor levels be removed?
+# @param na_action function defining how to treat NAs
+# @param drop_unused_levels should unused factor levels be removed?
 # @param attr_terms a list of attributes of the terms object of
 #   the original model.frame; only used with newdata;
 #   this ensures that (1) calls to 'poly' work correctly
@@ -11,7 +11,7 @@
 # @param knots: a list of knot values for GAMMs
 # @return model.frame for use in brms functions
 validate_data <- function(data, bterms, data2 = list(), knots = NULL,
-                          na.action = na.omit2, drop.unused.levels = TRUE,
+                          na_action = na_omit, drop_unused_levels = TRUE,
                           attr_terms = NULL) {
   if (missing(data)) {
     stop2("Data must be specified using the 'data' argument.")
@@ -20,7 +20,7 @@ validate_data <- function(data, bterms, data2 = list(), knots = NULL,
     knots <- get_knots(data)
   }
   data <- try(as.data.frame(data), silent = TRUE)
-  if (is(data, "try-error")) {
+  if (is_try_error(data)) {
     stop2("Argument 'data' must be coercible to a data.frame.")
   }
   if (!isTRUE(nrow(data) > 0L)) {
@@ -52,9 +52,9 @@ validate_data <- function(data, bterms, data2 = list(), knots = NULL,
   attr(data, "terms") <- NULL
   data <- model.frame(
     all_vars_terms, data, na.action = na.pass,
-    drop.unused.levels = drop.unused.levels
+    drop.unused.levels = drop_unused_levels
   )
-  data <- na.action(data, bterms = bterms)
+  data <- na_action(data, bterms = bterms)
   if (any(grepl("__|_$", colnames(data)))) {
     stop2("Variable names may not contain double underscores ",
           "or underscores at the end.")
@@ -67,6 +67,7 @@ validate_data <- function(data, bterms, data2 = list(), knots = NULL,
   data <- combine_groups(data, groups)
   data <- fix_factor_contrasts(data, ignore = groups)
   attr(data, "knots") <- knots
+  attr(data, "drop_unused_levels") <- drop_unused_levels
   data
 }
 
@@ -114,7 +115,7 @@ validate_data2 <- function(data2, bterms, ...) {
     attr(data2[[M]], "obs_based_matrix") <- TRUE
   }
   # validate within-group covariance matrices
-  cov_names <- ulapply(get_re(bterms)$gcall, "[[", "cov")
+  cov_names <- ufrom_list(get_re(bterms)$gcall, "cov")
   cov_names <- cov_names[nzchar(cov_names)]
   for (cov in cov_names) {
     data2[[cov]] <- validate_recov_matrix(get_from_data2(cov, data2))
@@ -263,6 +264,7 @@ subset_data <- function(data, bterms) {
     # cross-formula indexing is no longer trivial for subsetted models
     check_cross_formula_indexing(bterms)
     data <- data[subset, , drop = FALSE]
+    attr(data, "subset") <- subset
   }
   if (!NROW(data)) {
     stop2(
@@ -277,7 +279,7 @@ subset_data <- function(data, bterms) {
 }
 
 # like stats:::na.omit.data.frame but allows to certain NA values
-na.omit2 <- function(object, bterms, ...) {
+na_omit <- function(object, bterms, ...) {
   stopifnot(is.data.frame(object))
   nobs <- nrow(object)
   if (is.mvbrmsterms(bterms)) {
@@ -354,6 +356,10 @@ get_knots <- function(data) {
   attr(data, "knots", TRUE)
 }
 
+get_drop_unused_levels <- function(data) {
+  out <- attr(data, "drop_unused_levels", TRUE) %||% TRUE
+}
+
 # extract name of the data as originally passed by the user
 get_data_name <- function(data) {
   out <- attr(data, "data_name", TRUE)
@@ -390,7 +396,7 @@ validate_newdata <- function(
   incl_autocor = TRUE, group_vars = NULL, req_vars = NULL, ...
 ) {
   newdata <- try(as.data.frame(newdata), silent = TRUE)
-  if (is(newdata, "try-error")) {
+  if (is_try_error(newdata)) {
     stop2("Argument 'newdata' must be coercible to a data.frame.")
   }
   object <- restructure(object)
@@ -411,7 +417,7 @@ validate_newdata <- function(
     # variables not used in the included model parts
     # do not need to be specified in newdata
     resp <- validate_resp(resp, bterms$responses)
-    form_req_vars <- lapply(bterms$terms[resp], "[[", "allvars")
+    form_req_vars <- from_list(bterms$terms[resp], "allvars")
     form_req_vars <- allvars_formula(form_req_vars)
     req_vars <- intersect(req_vars, all.vars(form_req_vars))
   }
@@ -567,8 +573,8 @@ validate_newdata <- function(
   attr_terms <- c("variables", "predvars")
   attr_terms <- attributes(old_terms)[attr_terms]
   newdata <- validate_data(
-    newdata, bterms = bterms, na.action = na.pass,
-    drop.unused.levels = FALSE, attr_terms = attr_terms,
+    newdata, bterms = bterms, na_action = na.pass,
+    drop_unused_levels = FALSE, attr_terms = attr_terms,
     data2 = current_data2(object, newdata2),
     knots = get_knots(object$data)
   )
@@ -612,10 +618,13 @@ validate_newdata2 <- function(newdata2, object, ...) {
 }
 
 # extract the current data
-current_data <- function(object, newdata = NULL, ...) {
+current_data <- function(object, newdata = NULL, skip_validation = FALSE, ...) {
   stopifnot(is.brmsfit(object))
+  skip_validation <- as_one_logical(skip_validation)
   if (is.null(newdata)) {
     data <- object$data
+  } else if (skip_validation) {
+    data <- newdata
   } else {
     data <- validate_newdata(newdata, object = object, ...)
   }
@@ -623,10 +632,13 @@ current_data <- function(object, newdata = NULL, ...) {
 }
 
 # extract the current data2
-current_data2 <- function(object, newdata2 = NULL, ...) {
+current_data2 <- function(object, newdata2 = NULL, skip_validation = FALSE, ...) {
   stopifnot(is.brmsfit(object))
+  skip_validation <- as_one_logical(skip_validation)
   if (is.null(newdata2)) {
     data2 <- object$data2
+  } else if (skip_validation) {
+    data2 <- newdata2
   } else {
     data2 <- validate_newdata2(newdata2, object = object, ...)
   }
