@@ -9,8 +9,8 @@ opts_chunk$set(
   message = FALSE,
   warning = FALSE,
   eval = if (isTRUE(exists("params"))) params$EVAL else FALSE,
-  dev = "png",
-  dpi = 150,
+  dev = "jpeg",
+  dpi = 100,
   fig.asp = 0.8,
   fig.width = 5,
   out.width = "60%",
@@ -34,13 +34,13 @@ fit_imp1 <- brm_multiple(bmi ~ age*chl, data = imp, chains = 2)
 summary(fit_imp1)
 
 ## ---------------------------------------------------------------------------------------
-plot(fit_imp1, pars = "^b")
+plot(fit_imp1, variable = "^b", regex = TRUE)
 
 ## ---------------------------------------------------------------------------------------
 round(fit_imp1$rhats, 2)
 
 ## ---------------------------------------------------------------------------------------
-marginal_effects(fit_imp1, "age:chl")
+conditional_effects(fit_imp1, "age:chl")
 
 ## ---- results = 'hide', message = FALSE-------------------------------------------------
 bform <- bf(bmi | mi() ~ age * mi(chl)) +
@@ -49,5 +49,13 @@ fit_imp2 <- brm(bform, data = nhanes)
 
 ## ---------------------------------------------------------------------------------------
 summary(fit_imp2)
-marginal_effects(fit_imp2, "age:chl", resp = "bmi")
+conditional_effects(fit_imp2, "age:chl", resp = "bmi")
+
+## ---------------------------------------------------------------------------------------
+nhanes$se <- rexp(nrow(nhanes), 2)
+
+## ---- results = 'hide', message = FALSE, eval = FALSE-----------------------------------
+#  bform <- bf(bmi | mi() ~ age * mi(chl)) +
+#    bf(chl | mi(se) ~ age) + set_rescor(FALSE)
+#  fit_imp3 <- brm(bform, data = nhanes)
 
